package dhaverdLogs;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.file.Paths;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

public class DLogger {

    private static final String osSeparator = System.getProperty("file.separator");
    private static final String indent = " ".repeat(22);
    private String fileName;
    private boolean autoGenerateFileName;
    private static String logDir = "";
    private String[] unacceptableChars = {"\\", "/", ":", "*", "?", "\"", "<", ">", "|"};

    private boolean customLogDir;

    public DLogger(boolean autoGenerateFileName, String ...fileName) {
        if (fileName.length != 0){
            this.fileName = replaceUnacceptableChar(fileName[0]);
        }
        this.autoGenerateFileName = autoGenerateFileName;
    }

    public void setLogDir(String dir){
        logDir = dir;
        this.customLogDir = true;
    }

    private String replaceUnacceptableChar(String str){
        for (String character : unacceptableChars){
            str = str.replace(character, "_");
        }
        return str;
    }

    private String getCurrentTime(){
        String time;
        DateTimeFormatter dtf = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LocalDateTime now = LocalDateTime.now();
        time = "[" + dtf.format(now) + "] ";
        return time;
    }

    private File getLogFile(){
        if (autoGenerateFileName){
            setLogName("log");
        }
        String path = "";
        if (customLogDir){
            path = logDir + osSeparator + fileName;
        } else {
            path = Paths.get("").toAbsolutePath().toString() + osSeparator + fileName;
        }
        return new File(path);
    }

    private void setLogName(String name){
        String time = getCurrentTime();
        time = time.replace("[", "");
        time = time.replace("]", "");
        time = time.replace("-", ".");
        time = time.replace(":", "-");
        time = time.replace(" ", "_");
        fileName = time + name + ".log";
    }

    public void log(String context, String text, boolean addTime, boolean addIndent) throws IOException {
        File log = getLogFile();
        String finalText = "";
        if (addIndent){
            finalText += indent;
        }
        if (addTime){
            finalText += getCurrentTime();
        }
        finalText += "[" + context + "] ";
        finalText += text;
        finalText = finalText.replace("\n", "\n" + indent);
        FileWriter writer = new FileWriter(log, true);
        writer.write(finalText + "\r\n");
        System.out.println(finalText);
        writer.close();
    }
}
